/* jshint esversion: 6 */
const path = require('path');
const HtmlWebpackPlugin = require('html-webpack-plugin');
const webpack = require('webpack');

module.exports = {
    context: path.resolve(__dirname, 'src'),
    entry: {
        "main": [
            './main.ts'
        ],
        "styles": [
            './main.scss'
        ]
    },
    devtool: 'source-map',
    resolve: {
        extensions: ['.js', '.jsx', '.ts', '.tsx', '.html'],
    },
    module: {
        rules: [{
            enforce: 'pre',
            test: /webpack\.config\.js$/,
            include: path.resolve(__dirname),
            loader: 'eslint-loader',
            options: {
                useEslintrc: false,
                emitWarning: true,
                emitError: true,
                failOnWarning: true,
                failOnError: true,
                baseConfig: 'webpack',
                rules: {
                    indent: ['error', 4]
                },
            },
        }, {
            // Javascript
            enforce: 'pre',
            test: /\.jsx?$/,
            //include: path.resolve(__dirname, 'src'),
            loader: 'eslint-loader',
            options: {
                useEslintrc: false,
                emitWarning: false,
                emitError: true,
                failOnWarning: false,
                failOnError: true,
                baseConfig: 'angular',
                rules: {
                    indent: ['error', 4]
                },
                plugins: [
                    'angular',
                    'html',
                    'security',
                    'this',
                    'jquery',
                    'promise'
                ]
            },
        }, {
            test: /\.jsx?$/,
            exclude: /node_modules/,
            loader: 'babel-loader'
        }, {
            test: /\.tsx?$/,
            exclude: /node_modules/,
            loader: 'ts-loader'
        }, {
            test: /\.html$/,
            use: [
                'ngtemplate-loader?relativeTo=/accountant-ui/src',
                'html-loader'
            ]
        }, {
            test: /\.less$/,
            use: [
                'style-loader',
                'css-loader',
                'less-loader',
            ]
        }, {
            test: /\.css$/,
            use: [
                'style-loader',
                'css-loader',
            ]
        }, {
            test: /\.scss$/,
            use: [
                'style-loader',
                'css-loader',
                'sass-loader',
                'resolve-url-loader',
                'sass-loader?sourceMap'
            ]
        }, {
            test: /\.(png|woff|woff2|eot|ttf|svg)$/,
            loader: 'url-loader?limit=100000'
        }]
    },
    plugins: [
        new webpack.ProvidePlugin({
            "window.jQuery": "jquery"
        }),
        new webpack.ContextReplacementPlugin(
            /angular(\\|\/)core(\\|\/)@angular/,
            path.resolve(__dirname, './')
        ),
        new HtmlWebpackPlugin({
            title: 'Accountant',
            template: 'index.ejs',
            hash: false,
            inject: true,
            compile: true,
            minify: false,
            chunks: 'all'
        })
    ],
    output: {
        path: path.resolve(__dirname, 'build'),
        filename: '[name].bundle.js',
        chunkFilename: '[name].chunk.js'
        //publicPath: 'js'
    },
    devServer: {
        proxy: {
            '/api': {
                target: 'http://localhost:5000',
                secure: false
            }
        },
        hot: true,
        noInfo: false,
        quiet: false,
    }
};
